/************
 * 全局配置
 ************/
var GLOBAL = {
	//是否开启调试模板
	P_DEBUG : false,
	J_DEBUG : false
};

var _bUpdateFontSize = false;

/************
 * 基础函数
 ************/
(function(_aoNS) {
	// 给NodeList加上常用操作
	"forEach map reduce".split(/\s+/).forEach(function(fnName){
		NodeList.prototype[fnName] = NodeList.prototype[fnName] || Array.prototype[fnName];
	});

	// dom 样式 - from jQuery
	var rroot = /^(?:body|html)$/i,
		rmsPrefix = /^-ms-/,
		rdashAlpha = /-([\da-z])/gi,
		core_pnum = /[\-+]?(?:\d*\.|)\d+(?:[eE][\-+]?\d+|)/.source,
		rnumnonpx = new RegExp("^(" + core_pnum + ")(?!px)[a-z%]+$", "i"),
		rmargin = /^margin/,
		cssProps = {
			"myfloat": "cssFloat"
		},
		cssHooks = {
		},
		cssNormalTransform = {
			letterSpacing: 0,
			fontWeight: 400
		},
		fcamelCase = function (all, letter) {
			return (letter + "").toUpperCase();
		};

	var camelCase = function (string) {
		return string.replace(rmsPrefix, "ms-").replace(rdashAlpha, fcamelCase);
	};
	
	var vendorPropName = function (style, name) {
		if (name in style) {
			return name;
		}
	
		var capName = name.charAt(0).toUpperCase() + name.slice(1),
		origName = name,
		i = cssPrefixes.length;
	
		while (i--) {
			name = cssPrefixes[i] + capName;
			if (name in style) {
				return name;
			}
		}
	
		return origName;
	};
	
	var style = function (dom, name) {
		if (!dom || dom.nodeType === 3 || dom.nodeType === 8 || !dom.style) {
			return;
		}

		var ret, type, hooks,
		origName = camelCase(name),
		style = dom.style;
	
		if(origName == "float") {
			origName = "myfloat";
		}
	
		name = cssProps[origName] || (cssProps[origName] = vendorPropName(style, origName));
		hooks = cssHooks[name] || cssHooks[origName];

		if (hooks && "get" in hooks && (ret = hooks.get(elem, false, null)) !== undefined) {
			return ret;
		}

		return style[name];
	};

	var curCSS = function (elem, name) {
		var ret, width, minWidth, maxWidth,
		computed = window.getComputedStyle(elem, null),
		style = elem.style;
		if (computed) {
			ret = computed.getPropertyValue(name) || computed[name];
			if (ret === "") {
				ret = style(elem, name);
			}

			if (rnumnonpx.test(ret) && rmargin.test(name)) {
				width = style.width;
				minWidth = style.minWidth;
				maxWidth = style.maxWidth;
			
				style.minWidth = style.maxWidth = style.width = ret;
				ret = computed.width;
			
				style.width = width;
				style.minWidth = minWidth;
				style.maxWidth = maxWidth;
			}
		}
	
		return ret;
	};
	
	var css = function (dom, name) {
		var val, num, hooks,
		origName = camelCase(name);
	
		if(origName == "float"){
			origName = "myfloat";
		}
	
		name = cssProps[origName] || (cssProps[origName] = vendorPropName(dom.style, origName));
		hooks = cssHooks[name] || cssHooks[origName];

		if (hooks && "get" in hooks) {
			val = hooks.get(dom, true, extra);
		}

		if (val === undefined) {
			val = curCSS(dom, name);
		}
	
		if (val === "normal" && name in cssNormalTransform) {
			val = cssNormalTransform[name];
		}
	
		return val;
	};
	
	var offset = function (dom) {
		var docElem, body, win, clientTop, clientLeft, scrollTop, scrollLeft,
		box = {
			top: 0,
			left: 0
		},
		elem = dom,
		doc = elem && elem.ownerDocument;

		if (!doc) {
			return;
		}
		body = doc.body;
		docElem = doc.documentElement;

		if (typeof elem.getBoundingClientRect !== "undefined") {
			box = elem.getBoundingClientRect();
		}
		win = window;
		clientTop = docElem.clientTop || body.clientTop || 0;
		clientLeft = docElem.clientLeft || body.clientLeft || 0;
		scrollTop = win.pageYOffset || docElem.scrollTop;
		scrollLeft = win.pageXOffset || docElem.scrollLeft;
		return {
			top: box.top + scrollTop - clientTop,
			left: box.left + scrollLeft - clientLeft
		};
	};
	
	var offsetParentFunc = function (dom) {
		var offsetParent = dom.offsetParent || document.body;
		while (offsetParent && (!rroot.test(offsetParent.nodeName) && css(offsetParent, "position") === "static")) {
			offsetParent = offsetParent.offsetParent;
		}
		return offsetParent || document.body;
	};
	
	var replaceClassName = function(dom, oldClassName, newClassName) {
		dom.className = dom.className.replace(oldClassName, newClassName);
	};
	
	//--- 公开接口 ---
	
	// 时间函数
	_aoNS.now = function() {
		return +new Date;
	};
	
	// dom 操作
	_aoNS.dom = function(_asQueryStr, _aoDom) {
		return (_aoDom || document).querySelector(_asQueryStr);
	};

	_aoNS.doms = function(_asQueryStr, _aoDom) {
		return (_aoDom || document).querySelectorAll(_asQueryStr);
	};
	
	_aoNS.docElem = function() {
		return document.documentElement;
	};
	
	//属性
	_aoNS.attr = function(dom, attr) {
		return dom.getAttribute(attr);
	};
	
	//样式
	_aoNS.addClass = function(dom, className) {
		dom.className += " " + className;
	};
	
	_aoNS.removeClass = function(dom, className) {
		replaceClassName(dom, className, "");
	};
	
	_aoNS.replaceClass = replaceClassName;

	//样式
	_aoNS.css = css;

	//位置
	_aoNS.position = function (dom) {
		var offsetParent = offsetParentFunc(dom);
		var offsetobj = offset(dom);
		var parentOffset = rroot.test(offsetParent.nodeName) ? {
			top: 0,
			left: 0
		} : offsetParent.offset();
		
		offsetobj.top -= parseFloat(css(dom, "marginTop")) || 0;
		offsetobj.left -= parseFloat(css(dom, "marginLeft")) || 0;

		parentOffset.top += parseFloat(css(offsetParent, "borderTopWidth")) || 0;

		return {
			top: offsetobj.top - parentOffset.top,
			left: offsetobj.left - parentOffset.left
		};
	};
})(window);

/************
 * 页面调试 + 异常处理
 ************/
(function(_aoNS) {
	var _t = 0,
		_oCache = [];

	/*
	 * 调试接口
	 */
	function _console(_anType, _asText, _abIsLogFile) {
		//调用java的log
		if (_abIsLogFile) {
			//App.logFile(_asText);
		}
		else if (GLOBAL.J_DEBUG && _asText) {
			if(_anType == 1) {
				App.errorLog(_asText);
			} else {
				App.log(_asText);
			}
		}

		//显示在页面
		if (GLOBAL.P_DEBUG) {
			var _sId = "KsmAilcOnsoLe",
				_oDomConsole = dom("#" + _sId);

			//计算时间
			_t = (+new Date) - _t;
		
			if (!_oDomConsole && document.body) {
				_oDomConsole = document.createElement("div");
				_oDomConsole.id = _sId;
				//调试器样式
				_oDomConsole.style.cssText = "position:absolute;left:0;" + 
					"background:#000;color:#fff;" + 
					"opacity:0.8;font-size:12px!important;padding:10px!important;z-index:999;";
				//由于4.4需要加fillDiv在头部，所以console插在body1比较合适
				document.body.insertBefore(_oDomConsole, document.body.firstChild);
			}
	
			//先cache一下
			_asText && _oCache.push(
				//1为错误
				(_anType == 1 ? "<b style='color:red'>" : "") + 
					_asText + (_t < 10000 ? " t=" + _t + "ms" : "") +
				(_anType == 1 ? "</b>" : "")
			);

			//没创建成功的话，不处理
			if (_oDomConsole && _oCache.length > 0) {
				var _oDomDiv = document.createElement("div");
				//少于10s显示时间差
				_oDomDiv.innerHTML = _oCache.join("<br>");
				_oDomConsole.appendChild(_oDomDiv);
				//清空cache
				_oCache.length = 0;
			}

			//更新时间
			_t = +new Date;
		}
	}

	/*
	 * 打普通日志
	 */
	function _log(_asText) {
		_asText && _console(0, _asText);
	}

	/*
	 * 打错误日志，强制输出到文件
	 */
	function _err(_asText) {
		_asText && _console(1, _asText, true);
	}

	if (GLOBAL.P_DEBUG) {
		//保护：刷新出console数据
		document.addEventListener("DOMContentLoaded", function() {
			_console();
		});
	}
	
	//异常处理
	_aoNS.onerror = function(_asMsg, _asUrl, _asLine) {
		_err("js_error: msg=" + _asMsg + " url=" + _asUrl + " line=" + _asLine);
	};
	
	//默认显示调试器
	_log("-- Debug Opened --");

	//接口
	_aoNS.console = {
		log : _log,
		err : _err
	};
})(window);

//初始化KsMail JS 变量
var _nRestoreBodyScrollWidth = parseFloat(attr(docElem(), "contentWidth")) || 0,
	_nCurScale = parseFloat(attr(docElem(), "scale")) || 1,
	_nPageWidth = parseInt(attr(docElem(), "pageWidth")) || 360,
	_bIsShowImg = attr(docElem(), "showimage") == "false" ? false : true,
	_bIsAdMail = attr(docElem(), "admail") == "true" ? true : false,
	_bIsHasScaleTable = attr(docElem(), "tableFix") == "true" ? true : false,
	_bIsDarkMode = attr(docElem(), "darkMode") == "true" ? true : false,
	//某些机器很特别，刚开始的页面宽度较大导致缩放错误，需要延迟缩放
	//延迟缩放体验不是很好
	_bIsDelayScale = docElem().scrollWidth > _nPageWidth,
	_nPageMargin = 0,
	_bIsFixTable = _bIsHasScaleTable;

//	console.log(["rbsw", _nRestoreBodyScrollWidth, "scale", _nCurScale, "pw", _nPageWidth, "si", 
//		_bIsShowImg, "ad", _bIsAdMail, "delay", _bIsDelayScale].toString());

//图片操作函数
function _setImgMaxHeightBaseMaxWidth(_aoImg) {
	//按比例计算maxHeight
	var _nW, _nH;
	if ((_nW = _aoImg.naturalWidth) && (_nH = _aoImg.naturalHeight)) {
		//设置maxHeight
		_aoImg.style.maxHeight = (parseInt(_aoImg.width) * _nH / _nW)  + "px";
		//标记已经处理
		_aoImg.getAttribute("layoutResized", true);
	}
}
	
//设置缩放
function _setViewport(_anScale, _anWidth) {
	//12为右margin
	console.log("_setViewport:" + _anScale + "," + _anWidth);
	dom("#viewport").content = "initial-scale=" + _anScale + ",width=" + (_anWidth) + 
			" , user-scalable=yes,maximum-scale=10.0" + ", minimum-scale=" + _anScale;
}

//是否没有缩放状态
function _isScale() {
	return _nCurScale != 1 && !!_nRestoreBodyScrollWidth;
}


//调整图片缩放
function _fixImgScale() {
	replaceClass(dom("#mailcontent"), "mailcontent_beforescale", "mailcontent_scale");
}
	
//调整页面缩放
function _fixPageScale() {
	//页面缩放
	var _nBodyScrollWidth = document.body.scrollWidth;
	console.log("_fixPageScale bw=" + _nBodyScrollWidth + " pw=" + _nPageWidth + " rbsw=" + _nRestoreBodyScrollWidth);
	if (_nBodyScrollWidth > (Math.max(_nPageWidth, _nRestoreBodyScrollWidth))) {
		//边框为24
		/*//zmb添加begin
		var _ohtmlContent = dom("#mailcontent"),
		    _oTableAreas = doms(".kmTableArea", _ohtmlContent),
		    _nTableAreasLen = _oTableAreas.length;
		console.log("zmb _nTableAreasLen is:" + _nTableAreasLen);
		for (var i = 0; i < _nTableAreasLen; i++) {
			console.log("zmb after for:" +i);
			var _oDomImgs = doms("img", _oTableAreas[i]), table = _getTableFromTableArea(_oTableAreas[i]);
			console.log("zmb table.scrollWidth is:" + table.scrollWidth);
			if (table.scrollWidth > _nBodyScrollWidth) {
				_nBodyScrollWidth = table.scrollWidth;
				console.log("zmb _nBodyScrollWidth changed:" + table.scrollWidth);
			}
			var _oTDomImgs = doms("img", _oTableAreas[i]),
			imgLen = _oTDomImgs.length;
			console.log("zmb imgLen:" + imgLen);
			for (var i = 0; i < imgLen; i++) {
				console.log("zmb img width :" + _oTDomImgs[i].width);
				console.log("zmb img style width :" + _oTDomImgs[i].style.width);
			}
		}
		//zmb添加end*/
		var _nScale = _nPageWidth / ((_nRestoreBodyScrollWidth = _nBodyScrollWidth));
		//设置缩放
		_setViewport(_nCurScale = _nScale, _nRestoreBodyScrollWidth);
		//保存缩放比例
		//App.saveScale(_nScale, _nRestoreBodyScrollWidth);
		//返回缩放
		return true;
	}
	//返回没有缩放
	return false;
}
	
//重新设置表格区域的高度
function _resetTableAreaHeight(_aoDomArea) {
	var _nScale = parseFloat(attr(_aoDomArea, "nScale"));
	if (!isNaN(_nScale)) {
		_aoDomArea.style.cssText = "height:" +
			//用parseInt避免出现压缩后table与table之间出现空行 
			parseInt(_getTableFromTableArea(_aoDomArea).scrollHeight * _nScale + 
				(parseInt(_aoDomArea, "nOriHeightDelta") || 0)) + 
			"px!important;overflow:visible;"; 
		return true;
	}
	return false;
}

//获取页面显示内容的实际宽度
function _getPageContentWidth() {
    var mailContent = document.getElementById("mailcontent");
    var contentMarginLeft = parseInt(window.getComputedStyle(mailContent).getPropertyValue('margin-left'));
    var contentMarginRight = parseInt(window.getComputedStyle(mailContent).getPropertyValue('margin-right'));
    _nPageWidth = _nRestoreBodyScrollWidth - contentMarginLeft - contentMarginRight;
}

//进行表格缩放
function _fixTableScale(_aoDomArea, _aoDomTable, _anTableWidth) {
	//计算比例
	var _domAreaWidth = _aoDomArea.clientWidth;
	if(_domAreaWidth == 0) {
	    _domAreaWidth = _nPageWidth;
	}
	var _nScale = _domAreaWidth / _anTableWidth;
	console.log([_domAreaWidth, _anTableWidth, _nScale, parseFloat(attr(_aoDomArea, "nScale"))].toString());
	//需要缩放，而且跟当前缩放不一致
	if (_nScale >0 && _nScale < 1 && parseFloat(attr(_aoDomArea, "nScale")) != _nScale) {
	
		//计算偏移百分比(由于是中心缩放，缩放后需要平移到左上角位置
		var _nTranslate = (1 - _nScale) * 100 / (2 * _nScale);
		//设置样式
		
		_aoDomTable.style.webkitTransform =
			"scale(" + _nScale + ") translate(-" + _nTranslate + "%,-" + _nTranslate + "%)";
		//记录缩放大小
		_aoDomArea.setAttribute("nScale", _nScale);
		//由于缩放后，占位不变，外面高度也需要调整
		_resetTableAreaHeight(_aoDomArea);
		return true;
	}

	return false;
}

//是否能去掉容器内表格的宽带限制
function _canRemoveTablesWidth(_aoDomContainer) {
	var _nTrsLen = _aoDomContainer.getElementsByTagName("tr").length, 
		_nTdsLen = _aoDomContainer.getElementsByTagName("td").length, 
		_nThsLen = _aoDomContainer.getElementsByTagName("th").length,
		//表格是否接近单行 并且 不含有图片
		_bIsOk = _nTrsLen > 0 && (_nTdsLen > 0 || _nThsLen > 0) && 
			//安全起见，严格1
			(_nThsLen + _nTdsLen) / _nTrsLen == 1 &&
			//太危险了，table写100%的很容易会乱掉，再加个条件，table嵌套不允许
			//或者可以改用判断 width 是否 100% 
			_aoDomContainer.getElementsByTagName("table").length == 0;
	//console.log(["_nTrsLen", _nTrsLen, "_nTdsLen", _nTdsLen, "_nThsLen", _nThsLen, "_canRemoveTablesWidth ", _bIsOk].toString());
	return _bIsOk;
}

//获取区域的缩放table
function _getTableFromTableArea(_aoTableAreaDom) {
	//由于某些情况下，解释到的talbe前面有&nbsp;&nbsp;的情况。所以改用此方式获取较为安全。
	var _oTable = dom("table", _aoTableAreaDom);
	!_oTable && console.err("table not found in kmTableArea!");
	//table不存在的情况（理论上不会有，但是担心html错乱也有可能），做个保护避免js错误
	return _oTable || _aoTableAreaDom;
}

function _fixTableScaleReady(_aoDomArea) {
	//要过滤内area
//		if (getComputedStyle(_aoDomArea)["overflow"] != "visible") {
		//用firstChild 有可能不是 table
		var _oDomTable = _getTableFromTableArea(_aoDomArea),
			_nScrollWidth;
//			console.log("_canRemoveTablesWidth " + _canRemoveTablesWidth(_aoDomArea));
		//确定要fix的table才去掉margin
		addClass(_aoDomArea, "kmFixedTableArea" + 
			(_canRemoveTablesWidth(_aoDomArea) ? " kmFixedTableWidthForAll" : ""));
		//记录高度差（某些html会导致table前面会有&nbsp;导致存在高度差）
		_aoDomArea.setAttribute("nOriHeightDelta", _aoDomArea.scrollHeight - _oDomTable.scrollHeight);
		//必须要后于kmFixedTableWidthForAll的类来获取才能是正确的
		_aoDomArea.setAttribute("nOriWidth", _nScrollWidth = _oDomTable.scrollWidth);
//			console.log("zmb zmb zmb _nScrollWidth" + _nScrollWidth);
		//记录原宽度，方便refix的时候使用
		_fixTableScale(_aoDomArea, _oDomTable, _nScrollWidth);
//		}
}
	
//调整所有表格缩放
function _fixTablesScale(_aoDomAreas) {
	// _nLen > 0 && _fixTableScaleReady(_aoDomAreas[0]);
	// _nLen > 1 && setTimeout(function() {
	
		for (var i = 0, l = _aoDomAreas.length; i < l; i++) {
			_fixTableScaleReady(_aoDomAreas[i]);
		}
	// 	_aoDomAreas = null;
	// 	//因为有延迟，加入reset高度
	// 	_resetContentHeight();
	// });
}

//重新调整所有表格缩放
function _refixTablesScale() {
	console.log("_refixTablesScale " + document.body.clientWidth + " " + window.orientation);
	var _oDomContent = dom("#mailcontent"), 
		_oDomAreas = doms(".kmTableArea", _oDomContent);
	console.log(" _oDomAreas.length:" +  _oDomAreas.length);
	for (var i = 0, l = _oDomAreas.length, _nTableWidth, _oDomArea, _oDomTable; i < l; i++) {
		console.log("_refixTablesScale enter for ");
		if (!isNaN(_nTableWidth = parseInt(attr(_oDomArea = _oDomAreas[i], "nOriWidth")))) {
			if (!_fixTableScale(_oDomArea, _oDomTable = _getTableFromTableArea(_oDomArea), _nTableWidth) && 
					(parseFloat(attr(_aoDomArea, "nScale")) || 10) < 1) {
				console.log("_refixTablesScale enter for if");
				//去掉scale
				_oDomTable.style.webkitTransform = "";
				_oDomArea.setAttribute("nScale", 1);
				_resetTableAreaHeight(_oDomArea);
			}
		}
	}
}

//添加缩放事件
function _addScaleEvent() {
	//使用轮询方案加速触发时机
	var _nOrientationChangeTimer = 0,
		_nKSTimerStartTime = 2000000000000,
		_nBodyWidth = document.body.clientWidth;

	function _timerFunc() {
		//console.log("orientationchange _timerFunc");
		//运行时间过长，就不再探测，避免死循环
		if ((now() - _nKSTimerStartTime) > 2000) {
			console.log("resetContentHeight _addScaleEvent timeout");
			_clearInterval();
			return true;
		}
		
		var _nClientWidth = document.body.clientWidth;
		console.log("orientationchange" + document.body.clientWidth + " " + _nBodyWidth);

		if (Math.abs(_nClientWidth - _nBodyWidth) > 100) {
			console.log("orientationchange in if _nBodyWidth" + _nBodyWidth);
			_nBodyWidth = _nClientWidth;
			_clearInterval();
			_refixTablesScale();
			//_fixPageScale();
			_resetContentHeight();
			
			return true;
		}

		return false;
	}

	function _clearInterval() {
		if (_nOrientationChangeTimer) {
			clearInterval(_nOrientationChangeTimer);
			_nOrientationChangeTimer = 0;
		}
	}
	window.addEventListener("orientationchange", function() {
		console.log("orientationchange 123");
		_nKSTimerStartTime = now();
		if (!_timerFunc()) {
			_clearInterval();
			_nOrientationChangeTimer = setInterval(_timerFunc, 100);
		}
//			setTimeout(_timerFunc, 500);
	});
}
	
//为了异步调用
function _setLoadsImagesAutomatically() {
	//App.setLoadsImagesAutomatically();
}

function _getImgSetedWidth(_aoImg) {
	var _nWidth;
	if (_nWidth = _aoImg.oriWidth) {
		return _nWidth;
	}
	var _sWidth = _aoImg.style.width || attr(_aoImg, "width");
	if (_sWidth && _sWidth.indexOf("%") == -1) {
		return parseInt(_sWidth);
	}
	return null;
}

function _getImgSetedHeight(_aoImg) {
	var _nHeight;
	if (_nHeight = _aoImg.oriWidth) {
		return _nHeight;
	}
	var _sHeight = _aoImg.style.height || attr(_aoImg, "height");
	if (_sHeight && _sHeight.indexOf("%") == -1) {
		return parseInt(_sHeight);
	}
	return null;
}

function _reCalcImgHeight(_aoImg, _abIsAbsoluteValue) {
	var _nWidth, _nHeight, _nImgWidth;
	if (//_aoImg.style.display.toLowerCase() != "block" && 
		(_nWidth = _getImgSetedWidth(_aoImg)) && 
		(_nHeight = _getImgSetedHeight(_aoImg)) && 
		_nHeight > 30 && 
		_nWidth > (_nImgWidth = _aoImg.width)) {
		if (_abIsAbsoluteValue) {
			_aoImg.oriWidth = _nWidth;
			_aoImg.oriHeight = _nHeight;
			_aoImg.style.height = (_nHeight * _nImgWidth / _nWidth) + "px";
			console.log("ANGUS_WEBVIEW _initImages w=" + _nWidth + " h=" + _nHeight + " value=" + (_nHeight * _nImgWidth / _nWidth));
		}
		else {
			_aoImg.style.cssText = "height:auto!important;";
		}
	}
}

//
function _initImages(_aoImg) {
	//所有img的图片处理
	var _oImgs = /*doms("img")*/_aoImg, _sSrc;
	for (var i = 0, _nLen = _oImgs.length, _oImg, _sPath; i < _oImgs.length; i++) {
		//设置最大宽度
		_oImg = _oImgs[i];
		//zmb  _sSrc = _oImg.src.split("#")[0];
		_sPath = _oImg.src;//zmb _sSrc && App.getImageCachePath(_oImg.id || (_oImg.id = "img_" + i), _sSrc);
		//console.log("_showImages getImageCachePath id=" + _oImg.id + " src=" + _oImg.src + " path=" + _sPath);//.split("#")[0]
		//对于某些段落加了 text-indent max-width=100% 会有问题。
		//这里可以去掉，但是会影响性能等等，与其他效果，还是先不修改
		//_oImg.parentNode.style.textIndent = 0;
		if (_bIsShowImg && _sPath) {
			//## Problom1 ##
			//Neuxs4 Android4.4 Webkit 下有点奇怪 
			//在初始化的时候，设置相同src，只有最后那个img成功显示图片。
			console.log("zmb load Image:" + _oImg.width);
			_loadImage(_oImg, _sPath);//zmb (_oImg, _sPath + "?r=" + i); 
		}
		else {
			//存在宽高且较大才显示边框
			//console.log("w=" + _oImg.width + " h=" + _oImg.height);
			var _nWidth = _oImg.width,
				_nHeight = _oImg.height;
			if (_nWidth > 50 && _nHeight > 50) {
				addClass(_oImg, "ksmail_img_spacer");
			}
			else if (_nWidth == 0 && _nHeight == 0) {
				addClass(_oImg, "ksmail_img_spacer_wh")
			}
			//参看 ## Problom1 ##
			//zmb _oImg.src = "file:///android_asset/spacer.png?r=" + i;
			//console.log(_oImg.id + " ksmail_img_spacer");
			//存在高度的图片，都可能需要重新计算
			_reCalcImgHeight(_oImg, true);
		}
	 }
}

//图片加载完成的处理逻辑
function _imgOnLoad() {
	//不需要fixtable就不做了
	console.log("图片加载完成的处理逻辑");
	if (_bIsFixTable) {
		console.log("需要fixtable就做了");
		//根据maxWidth调整maxHeight
		//_setImgMaxHeightBaseMaxWidth(this);
		//可能图片存在table中，所以需要调整一下对应的table的高度
		//zmb add begin
		var _oNode2 = this.parentNode;
		console.log("_oNode.tagName:" + _oNode2.tagName);
		//zmb add end
		for (var _oNode = this.parentNode, _nOriWidth; _oNode && _oNode.tagName != "BODY"; _oNode = _oNode.parentNode) {
			console.log("nOriWidth=" + attr(_oNode, "nOriWidth") + " parseInt=" + parseInt(attr(_oNode, "nOriWidth")));
			if (!isNaN(_nOriWidth = parseInt(attr(_oNode, "nOriWidth")))) {
				var _oDomTable = _getTableFromTableArea(_oNode),
					_nScrollWidth = _oDomTable.scrollWidth;
				console.log("imgOnLoad ow=" + _nOriWidth + " sw=" + _nScrollWidth);
				//如果图片撑大了表格，需要重新缩放.否则调整高度则可。
				if (_nOriWidth != _nScrollWidth) {
					console.log("图片撑大了表格，需要重新缩放");
					_oNode.setAttribute("nOriWidth", _nScrollWidth);
					_fixTableScale(_oNode, _oDomTable, _nScrollWidth);
				}
				//否则改变高度即可
				else {
					console.log("否则改变高度即可");
					_resetTableAreaHeight(_oNode);
				}
				break;
			}
		}
	}
//	//由于table内的图片没有max-width，所以要再fixPage一下
//	else if (_fixPageScale()){
//		//更新scale后需要update一下，否则4.4有问题
//		updateTitleBar();
//	}
	//android webview 加载图片后由时候不会reflow&repaint，强制处理一下
	reflowAndRepaint();
	//图片加载成功后需要更新正文区高度发生变化，以保证附件位置正确性
	_resetContentHeight();
}

//加载单张图片
function _loadImage(_aoImg, _asSrc) {
	//相同的图片就不多加载了
	console.log("__loadImage 加载单张图片");
	//if (_asSrc != _aoImg.src) { //zmb打开的
		console.log("__loadImage 加载单张图片 进来了");
		_aoImg.onload = _imgOnLoad;
		//console.log("_loadImage id=" + _aoImg.id + " src=" + _asSrc);
		_aoImg.src = _asSrc;
		//这个方法还是有bug的。最后是直接设置值，但是resize的时候可能有问题
		_reCalcImgHeight(_aoImg);
	//} //zmb打开的
}

//图片显示入口
function _showImages() {
	//console.log("ANGUS_WEBVIEW _bIsShowImg=" + _bIsShowImg);
	if (_bIsShowImg) {
		//显示背景图片
		removeClass(dom("#mailcontent"), "ksmail_nobgimg");
		//放开webview加载图片
		//必须异步，否则上面的src虽然设置了，但是实际js没那么快flash，渲染进程有可能发旧的链接出去
		//setTimeout(_setLoadsImagesAutomatically);
		//貌似不需要异步也ok？
		_setLoadsImagesAutomatically();
	}
}

//bug47189 每行td由两张图拼凑而成，图片太大导致换行，显示错乱
function _updateImageWidthForTD(_oDomContent, _oDomImgs) {
    if(_oDomImgs.length < 2) {
        return;
    }
    var tds = _oDomContent.getElementsByTagName("td");
    var i = 0, j = 0;
    for(i = 0;i < tds.length;i++) {
        var td = tds[i];
        var childTd = td.getElementsByTagName("td");
        //获取每根树枝上最后一个td节点
        if(childTd.length > 0) {
            continue;
        }
        var images = td.getElementsByTagName("img");
        if(images.length >= 2) {
            var imageMaxWidth = _nPageWidth / images.length;
            for(j = 0;j < images.length;j++) {
                var display = images[j].style.display;
                if(display === "inline") {
                    images[j].style.width = imageMaxWidth+"px";
                }
            }
        }
    }
}

//调整缩放
function _fixScale(_abIsRecordScale) {
	var _oDomContent = dom("#mailcontent"),
		_oDomImgs = doms("img", _oDomContent);
	//console.log("_fixScale " + _bIsHasScaleTable);
	//已经保存缩放表格数据的话，不需要再缩放表格
	if (!_bIsHasScaleTable) {
		//先判断是否需要表格缩放
		var _oDomAreas = doms(".kmTableArea", _oDomContent),
			_nDomAreasLen = _oDomAreas.length,
			_oDomFixAreas = [],
			_nTableImgs = 0;
		var _tableMaxWidth = 0;
		for (var i = 0, _oDomArea; i < _nDomAreasLen; i++) {
			_oDomArea = _oDomAreas[i];
			if(!isNaN(_oDomArea.scrollWidth) && _tableMaxWidth < _oDomArea.scrollWidth) {
			    _tableMaxWidth = _oDomArea.scrollWidth;
			}
			if (getComputedStyle(_oDomArea)["overflow"] != "visible") {
				_oDomFixAreas.push(_oDomArea);
				_nTableImgs += doms("img", _oDomArea).length;
			} else if(_bUpdateFontSize) {
			     _oDomFixAreas.push(_oDomArea);
			}
		}
		//"word-break"属性强制元素换行，解决BUG #47255表格太长显示成一条线的BUG
		if(_tableMaxWidth > 3000) {
		    document.body.removeAttribute("word-wrap");
		    document.body.style.wordBreak = "break-word";
		}
		 _bUpdateFontSize = false;
		//console.log(["ad", _bIsAdMail, "tableimgs", _nTableImgs, "imgs", _oDomImgs.length, " domAreaLen", _nDomAreasLen].toString());
		if ((_bIsAdMail && _nTableImgs > 0) || (_nTableImgs * 2 > _oDomImgs.length && _nTableImgs != _oDomImgs.length) || _nDomAreasLen == 0) {			
			addClass(_oDomContent, "kmNoTableScale");
			console.log("_fixScale 未缩放表格");
			//不需要保存内容，浪费空间（某些有表格情况，需要去掉kmTableArea的样式，所以要kmNoTableScale配合）
			//App.saveModifyContent("<!--NotFixTable-->");
		}
		else {
			console.log("_fixScale 开始缩放表格");
			//记录当前页面需要fixtable
			_bIsFixTable = true;
			//提高显示速度，避免看到跳动感，把fixTablesScale提前
			//调整表格最大宽度
			_fixTablesScale(_oDomFixAreas);
			//保存内容
			//App.saveModifyContent(_oDomContent.innerHTML);
		}
		console.log("finish modify table");
	}
	_updateImageWidthForTD(_oDomContent, _oDomImgs);
	//如果没有保留缩放比例，先计算缩放，再缩放图片
	//如果已经保留缩放比例，那么不用再计算，直接缩放图片即可
	console.log("before _fixPageScale");
	!_abIsRecordScale && _fixPageScale();
	//4.4以上，需要通知一下改变titleBar，以便全部显示正文
	//这个依赖于_fixPageScale的数据改变
	//updateTitleBar();
	//修正JellyBean下实现titleBar带来的跳动
	//App.fixJellyBeanTitleBar();
	//调整图片最大宽度
	_fixImgScale();
	//图片处理异步化
	// setTimeout(function() {
		//初始化图片的大小（由于max-width的设置，可能导致设置高度的图片比例出错，所以这里需要都计算一下）
		//图片高度有可能影响table高度，所以先做这个再缩放表格
		_initImages(_oDomImgs);
		//缩放后，显示图片
		_showImages();
		//异步后要重新调整高度
		// _resetContentHeight();
	// }, 0);
		console.log("document.body.scrollWidth:" + document.body.scrollWidth);
	//缩放完成后，添加事件，以便改变也可以重新缩放
	_addScaleEvent();

	//告知webview内容高度变化
	//第一次调用给某些机器加速
	//放上面了，这个就不用了？
	setTimeout(_resetContentHeight, 0);
	//过快回调，java获取到页面的scale的值还是旧的，导致content-height 的值计算错误，附件位置会出错
	setTimeout(_resetContentHeight, 400);
}

//缩放入口
function _startFixScale(){
	
	var _bIsScale = _isScale();
	//console.log("_bIsScale=" + _bIsScale + " _bIsDelayScale=" + _bIsDelayScale);
	//由于默认宽度过大，需要延迟后宽度才对
	if(true) {
		//延迟0ms就可以（这个测试过，不知道有无其它机器还要调大）
		setTimeout(_fixScale, 100);
	}
	else {
		_fixScale(_bIsScale);
	}
}

function _resetContentHeight() {
	var _oDomFildDiv = dom("#fillDiv");
	if (!_oDomFildDiv) {
		return console.err("#fillDiv not found!");
	}
	var _nCurFillDivTop = position(_oDomFildDiv).top;
	//高度没变化就不回调了，减少回调次数
	if (_resetContentHeight._nOldFillDivTop != _nCurFillDivTop) {
		//console.log("fillDiv top=" + _nCurFillDivTop);
		//有缩放的情况下，需要通知client更新fillDiv以便支持小内容的滚动以及附件位置
		//App.setRealContentHeight(_resetContentHeight._nOldFillDivTop = _nCurFillDivTop);
		//通知页面更新缩放
		//App.resetContentHeight();
	}
}

function _showToolBar() {
	//console.log("_showToolBar");
	//App.showToolBar();
}

var _bIsFirstSetBar = true;
//调整FillDiv高度，公开给java调用
window.setBarFillDivHeight = function(_anFillDivHeight, _anToolBarHeight) {
	//第一次设置完成后才显示toolbar
	if (_bIsFirstSetBar) {
		//附件需要设置该值后才能正确显示对应位置，所以设置在这里
		//console.log("setBarFillDivHeight first");
		_bIsFirstSetBar = false;
		setTimeout(_showToolBar, 100);
	}
	//console.log("func-setBarFillDivHeight fillDivHeight=" + _anFillDivHeight + " toolBarHeight=" + _anToolBarHeight);
	dom("#fillDiv").style.height = _anFillDivHeight +"px";
	dom("#toolBarFillDiv").style.height = _anToolBarHeight +"px";
};

//显示图片，公开给java调用
window.loadImage = function(_asId, _asPath) {
	console.log("window.loadImage " + _asId + ", path:" + _asPath);
	var _oImg = dom("#" + _asId);
	removeClass(_oImg, "ksmail_img_spacer");
	_loadImage(_oImg, _asPath);
};

//显示多张图片，公开给java调用
window.loadImages = function(_asIds, _asPath) {
	console.log("window.loadImages " + _asIds + ", path:" + _asPath);
	var _oIds = _asIds.split(",|,");
	for (var i = 0, l = _oIds.length; i < l; i++) {
		loadImage(_oIds[i], _asPath, true);
	}
}

//打开图片加载，公开给java调用
window.resetShowImages = function() {
	if (!_bIsShowImg) {
		_bIsShowImg = true;
		_showImages();
	}
}

//公开给java层调用reflow&repaint
window.reflowAndRepaint = function() {
	document.body.style.overflow = "hidden";
	document.body.offsetHeight;
	document.body.style.overflow = "auto";
	console.log("reflowAndRepaint");
}

//公开给java层调用显示正在加载全部
window.showLoadingAll = function() {
	dom("#ksmailLoadAllContent").style.display = "none";
	dom("#ksmailLoadingAllContent").style.display = "";
}

// 4.4以上的titleBarWebView 使用到
var _nCurTitleBarHeight = 0,//App.getTitleBarHeight(),
	//跟getTitleBarHeight()一致，以便下面判断减少1次
	_nDisplayTitleBarHeight = -1,
	_nDisplayScale;
// 公开给java调用
window.updateTitleBar = function(_anTitleBarHeight) {
	//存在数据，更新titleBar高度值
	if (_anTitleBarHeight > 0) {
		_nCurTitleBarHeight = _anTitleBarHeight;
	}
	//参数发生改变的话，重置高度
	if (_nCurTitleBarHeight > 0 && (_nCurTitleBarHeight != _nDisplayTitleBarHeight || _nDisplayScale != _nCurScale)) {
		var _sId = "ksmailTitleBarFillDiv",
			_oDom = dom("#" + _sId);
		//容器不存在就自己创建
		if (!_oDom) {
			_oDom = document.createElement("div");
			_oDom.id = _sId;
			docElem().insertBefore(_oDom, document.body);
		}
		//设置高度
		_oDom.style.height = (_nCurTitleBarHeight / _nCurScale) + "px";
		//记录显示状态，以便优化避免重复设置
		_nDisplayTitleBarHeight = _nCurTitleBarHeight;
		_nDisplayScale = _nCurScale;
		//
		//console.log("updateTitleBar " + _anTitleBarHeight);
	}
};

// 公开给Java更新
window.updateSelection = function() {
	var selected = window.getSelection();
	//App.selectionRange(selected.toString().length);
	//console.log("getSelection : " + selected.rangeCount + ", " + selected.toString().length);
}

// 初始化scale
if (_isScale())  {
	// 究竟是否需要？ 除了 contentWidth + 30 没有被设置
	_setViewport(_nCurScale, _nRestoreBodyScrollWidth);
}

(function(){   
	// 监控选择文字状态
	changeDarkModeForHead();
	document.addEventListener("selectionchange", function() {
		updateSelection();
	}, false);

	document.addEventListener("DOMContentLoaded", function() {
		//开始缩放
//		if (document.getElementById("isAD") == null) {
//        			removeFontBackGroundCss();
//        }
        changeDarkMode();
        _getPageContentWidth();
		_startFixScale();
		_setImageLongClickListener();
		autoFormat();
		updateNeteaseBigAttachmentDivs();
	}, false);
})();
function updateNeteaseBigAttachmentDivs() {
    var attachmentDivs = document.querySelectorAll("#divNeteaseBigAttach");
    if (attachmentDivs) {
        for (var i = 0; i < attachmentDivs.length; i++) {
            var attachmentDiv = attachmentDivs[i];
            var background = attachmentDiv.style.background;
            attachmentDiv.style.background = 'inherit';

            if (attachmentDiv) {
                var allDivs = attachmentDiv.querySelectorAll("div");
                for (var j = 0; j < allDivs.length; j++) {
                    if (j === 0) {
                        allDivs[j].style.background = "rgb(222, 232, 242)";
                        allDivs[j].style['padding-top'] = '8px';
                        allDivs[j].style['padding-bottom'] = '5px';
                        allDivs[j].style['padding-left'] = '5px';
                    }

                    //清除line-height数值
                    if (allDivs[j].style['line-height'] === '14px') {
                        allDivs[j].style['line-height'] = 'normal';
                    }

                    if (allDivs[j].style.height === '36px') {
                        allDivs[j].style.height = 'auto';
                        allDivs[j].style['margin-top'] = '10px';
                        allDivs[j].style['margin-bottom'] = '10px';

                    }

                    //设置附件信息左半部分
                    if (allDivs[j].style.width === '36px') {
                        if (j < allDivs.length - 1) {
                            allDivs[j + 1].style['margin-left'] = '40px'
                        }
                        allDivs[j].style['margin-top'] = '5px';
                    }
                }
            }
        }
    }
}

function changeDarkModeForHead() {
    if(!_bIsDarkMode) {
        return;
    }
    var styleElement = document.createElement("style");
    styleElement.innerHTML = "body{background-color: #000000 !important}";
    document.getElementsByTagName("head")[0].appendChild(styleElement);
}

function changeDarkMode() {
    if(!_bIsDarkMode) {
        return;
    }
    document.body.style.backgroundColor = "#000000";

    var walker = document.createTreeWalker(document, NodeFilter.SHOW_ELEMENT, {
        acceptNode: function(node) {
          if (ignoreDarkModeNode(node)) {
              return NodeFilter.FILTER_REJECT;
          }
          return NodeFilter.FILTER_ACCEPT;
        }
      }, false);

    var node = walker.nextNode();
     while(node) {
        if(node.tagName == "A") {
            node.style.setProperty("color", "#08a16d", "important");
            node = walker.nextNode();
            continue;
        }
        var backgroundValue = node.getAttribute("background");
        if(backgroundValue != null && backgroundValue.length > 0 && backgroundValue.length < 20) {
            node.setAttribute("background", "#000000");
        }
        var bgColorValue = node.getAttribute("bgcolor");
        if(bgColorValue != null && bgColorValue.length > 0) {
            node.setAttribute("bgcolor", "#000000");
        }
        var backgroundStyleValue = node.style.getPropertyValue("background");
        if(backgroundStyleValue != null && backgroundStyleValue.length > 0 && backgroundStyleValue.length < 20) {
            node.style.setProperty("background", "#000000", "important");
        }
        var backgroundColorValue = node.style.getPropertyValue("background-color");
        if(backgroundColorValue != null && backgroundColorValue.length > 0) {
            node.style.setProperty("background-color", "#000000", "important");
        }
        node.style["color"] = "#FFFFFF";
        node.style["border-color"] = "#FFFFFF";
        node = walker.nextNode();
     }
}

function removeFontBackGroundCss() {
	var _oDomContent = dom("#mailcontent");
	if(_oDomContent == null){
		return;
	}
	if(_oDomContent.innerHTML.length > 500000){
		return;
	}
	var themeColor = document.getElementById("ThemeColor");
	if (themeColor != null) {
	    if(document.styleSheets[0].cssRules != null && document.styleSheets[0].cssRules.length > 0) {
	    	document.styleSheets[0].cssRules[0].style["color"] = themeColor.innerHTML;
		}
		var quotebtn = document.getElementById("quotebtn");
		if (quotebtn != null) {
			quotebtn.style["color"] = "#9c9c9c";
		}
		var tables = document.getElementsByTagName("table");
        for (var i = 0; i < tables.length; i++) {
        	tables[i].style["background-color"] = "#ffffff";
        }
        var bodys = document.getElementsByTagName("body");
        for (var i = 0; i < bodys.length; i++) {
        	bodys[i].setAttribute("bgcolor", "");
        	bodys[i].setAttribute("text", "");
		}
	}
	var styleCount = document.styleSheets.length;
	var style, cssCount, cssRule;

	for (var i = 1; i < styleCount; i++) {
		style = document.styleSheets[i];
		var isHasCasRules = style.hasOwnProperty("cssRules");
		if(isHasCasRules && style.cssRules != null) {
		    cssCount = style.cssRules.length;
        		for (var j = 0; j < cssCount; j++) {
        			cssRule = style.cssRules[j];
        			if(cssRule != null && cssRule.style != null){
        				cssRule.style["font-size"] = "";
        				cssRule.style["text-indent"] = "";
        				cssRule.style["margin"] = "";
        				cssRule.style["margin-left"] = "";
        				cssRule.style["margin-right"] = "";
        				cssRule.style["margin-top"] = "";
        				cssRule.style["margin-bottom"] = "";
        				cssRule.style["padding"] = "";
        				cssRule.style["padding-left"] = "";
        				cssRule.style["padding-right"] = "";
        				if (themeColor != null) {
        					cssRule.style["color"] = "";
        					cssRule.style["background-color"] = "";
        					cssRule.style["background"] = "";
        				}
        			}
        		}
		}
	}

    var walker = document.createTreeWalker(document, NodeFilter.SHOW_ELEMENT, {
        acceptNode: function(node) {
          if (ignoreTNode(node, themeColor == null)) {
            return NodeFilter.FILTER_REJECT;
          }
          return NodeFilter.FILTER_ACCEPT;
        }
      }, false);

    var node = walker.nextNode();
    while(node) {
    	node.style["font-size"] = "";
		node.style["text-indent"] = "";
    	node.style["margin"] = "";
    	node.style["margin-left"] = "";
    	node.style["margin-right"] = "";
    	node.style["margin-top"] = "";
    	node.style["margin-bottom"] = "";
    	node.style["padding"] = "";
    	node.style["padding-left"] = "";
    	node.style["padding-right"] = "";
		if (themeColor != null) {
			node.style["color"] = "";
			node.style["background-color"] = "";
			node.style["background"] = "";
			node.removeAttribute("color");
		}
		node.removeAttribute("size");
    	node = walker.nextNode();
    }
	/*var obj = _oDomContent.getElementsByTagName("*");
	var objStyle;
	for (var i = 0; i < obj.length; i++) {
		objStyle = obj[i];
		if (objStyle.id == "quotebtn") {
			return;
		}
		objStyle.removeAttribute("size");
		objStyle.style["font-size"] = "";
		objStyle.style["text-indent"] = "";
		objStyle.style["margin"] = "";
		objStyle.style["margin-left"] = "";
		objStyle.style["margin-right"] = "";
		objStyle.style["margin-top"] = "";
		objStyle.style["margin-bottom"] = "";
		objStyle.style["padding"] = "";
		objStyle.style["padding-left"] = "";
		objStyle.style["padding-right"] = "";
	}*/
}

function ignoreTNode(node, isNormal) {
	var result = node.tagName == "TABLE" || node.id == "quotebtn" || node.id == "quotediv";
	if (isNormal) {
		result = result || node.tagName == "A";
	}
	return result;
}

function ignoreDarkModeNode(node) {
    return node.tagName == "DIV" && node.className == "max-body";
}

function updateFontSize() {
	_bUpdateFontSize = true;
	_startFixScale(dom("#mailcontent"));
}

function autoFormat() {
  var content = document.getElementById('mailcontent');
  var ret = linkify(content, [
		  emailTransform,
		  urlTransform,
		  phoneTransform
		  ]);
}

function phoneTransform(content) {
  phone.lastIndex = 0;
  return content.replace(phone, function(m) {
    var c = 0;
    for (var i = 0, l = m.length; i < l; i++) {
      if (/\d/.test(m.charAt(i))) c++;
    }
    if (c >= 5 && c <= 20) {
      var arr = m.split(/(\s|-)/);
      arr = arr.slice(arr.length - 3);
      if (arr.length > 0) {
        for (var i = 0, l = arr.length; i < l; i++) {
          if (arr[i].length > 2) {
            return '<a href="tel:' + m + '">' + m + '</a>';
          }
        }
      } else {
        return '<a href="tel:' + m + '">' + m + '</a>';
      }
    }
    return m;
  });
}

function emailTransform(content) {
  email.lastIndex = 0;
  return content.replace(email, '<a href="mailto:$&">$&</a>');
}

function urlTransform(content) {
  url.lastIndex = 0;
  return content.replace(url, function(m) {
    if (/^http/.test(m)) {
      return '<a href="' + m + '">' + m + '</a>';
    }
    return '<a href="http://' + m + '">' + m + '</a>';
  });
}

var IGNORE_NODE_NAMES = ['A', 'STYLE'];
function ignoreNode(node) {
  return node.parentNode &&
    (IGNORE_NODE_NAMES.indexOf(node.parentNode.nodeName.toUpperCase()) != -1
     || node.parentNode.hasAttribute('ignore'));
}

var phone = /(\+[0-9]+[\- ]*)?(\([0-9]+\)[\-]*)?([0-9][0-9\-][0-9\-]+[0-9]{2,})(?!\.)(?!\d)/g;
var email = /[a-zA-Z0-9\+\.\_\%\-\+]{1,256}\@[a-zA-Z0-9][a-zA-Z0-9\-]{0,64}(\.[a-zA-Z0-9][a-zA-Z0-9\-]{0,25})+/g;
var url = /((?:https?:\/\/|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}\/)(?:(&amp;amp;)|(&amp;)|[^\s`!()\uff08\uff09\[\]{};'"<>\u201c\u201d\u2018\u2019\u3002\uff0c\uff1b]|([(\uff08][^\s()\uff08\uff09<>]+[)\uff09])|([(\uff08][^\s()\uff08\uff09<>]*?[(\uff08][^\s()\uff08\uff09<>]*?[)\uff09][)\uff09]))+(?:([(\uff08][^\s()\uff08\uff09<>]+[)\uff09])|([(\uff08][^\s()\uff08\uff09<>]*?[(\uff08][^\s()\uff08\uff09<>]*?[)\uff09][)\uff09])|[^\s`!()\uff08\uff09\[\]{};:'".,<>?\u201c\u201d\u2018\u2019\u3002\uff0c\uff1b]))/g;

function linkify(el, transforms) {
  var node, nodes = [],
      count = 0, loop = 0, match = 0;

  var walker = document.createTreeWalker(el, NodeFilter.SHOW_TEXT, {
    // Android 2.3 Webkit 533.1 BUG:
    // https://bugs.webkit.org/show_bug.cgi?id=35296
    acceptNode: function(node) {
      if (ignoreNode(node)) {
        return NodeFilter.FILTER_REJECT;
      }
      return NodeFilter.FILTER_ACCEPT;
    }
  }, false);

  while (node = walker.nextNode()) {
      var nodePt = node.parentElement;
        if(nodePt.className != 'normal-text'){
            nodes.push(node);
        }
  }

  while (node = nodes.shift()) {
    count++;
    (function transform(node, i) {
      i = i || 0;
      var originalText = node.textContent;
      if (!transforms[i] || originalText.trim().length == 0) return;
      if (ignoreNode(node)) return;
      loop++;
      var newText = transforms[i](originalText);
      if (originalText != newText) {
        match++;
        var newNode = document.createElement('span');
        newNode.innerHTML = newText;
        node.parentNode.insertBefore(newNode, node);
        node.parentNode.removeChild(node);
        linkify(newNode, transforms);
      } else {
        transform(node, i+1);
      }
    })(node);
  }
  return [count, loop, match];
}

//YangZhenNan 引文使用
function change(contentId){
	var content = document.getElementById(contentId);
	content.style.display = "block";
	_startFixScale();
}

// liutz,更新内联图片的SRC
function updateSrc(cid, src) {
	var objs = document.getElementsByTagName("img");
	for (var i = 0; i < objs.length; i++) {
		if (objs[i].src.substr("cid:".length) == cid || objs[i].getAttribute("data-cid") == cid) {
			objs[i].src = src;
			objs[i].setAttribute("data-cid",cid);
		}
	}
}

// liutz,添加图片点击事件
function _setImageLongClickListener() {
	var objs = document.getElementsByTagName("img");
	for(var i=0;i<objs.length;i++) {
		objs[i].onclick=function() {
			imagelongclicklistener.onClick(this.src, this.getAttribute('data-cid'));
		};
	}
}

/**
1.安装Node.js
2.安装uglify-js
npm install -g uglify-js
3.切换至工程目录js目录下，执行uglifyjs命令
cd ${ProjectHome}/AndroidMail/assets/js
uglifyjs ksMailView.js -c -m reserved=['updateSrc','change','linkify'] -o ksMailView_mixed.js
混淆时请注意：zhennan关于引文的这个函数请不要混淆
混淆完成后，请把相关代码粘贴到ksMialView_mixed.js中 并进行测试 
*/